@@ -42,16 +42,27 @@ import abc
 import collections
 import datetime
 import decimal
+import hashlib
 import json
 import math
 import uuid
 import warnings
+from functools import reduce
 from pathlib import Path
-from typing import Mapping, MutableMapping, Optional, Sequence, Union, cast
+from typing import (
+    Callable,
+    FrozenSet,
+    List,
+    Mapping,
+    MutableMapping,
+    Optional,
+    Sequence,
+    Union,
+    cast,
+)
 
 import avro.constants
 import avro.errors
-from avro.constants import NAMED_TYPES, PRIMITIVE_TYPES, VALID_TYPES
 from avro.name import Name, Names, validate_basename
 
 #
@@ -104,6 +115,50 @@ def _is_timezone_aware_datetime(dt: datetime.datetime) -> bool:
     return dt.tzinfo is not None and dt.tzinfo.utcoffset(dt) is not None
 
 
+# Fingerprint Constants
+_EMPTY64_FINGERPRINT: int = 0xC15D213AA4D7A795
+_FINGERPRINT_TABLE: tuple = tuple(reduce(lambda fp, _: (fp >> 1) ^ (_EMPTY64_FINGERPRINT & -(fp & 1)), range(8), i) for i in range(256))
+
+
+# All algorithms guaranteed by hashlib are supported:
+#     - 'blake2b',
+#     - 'blake2s',
+#     - 'md5',
+#     - 'sha1',
+#     - 'sha224',
+#     - 'sha256',
+#     - 'sha384',
+#     - 'sha3_224',
+#     - 'sha3_256',
+#     - 'sha3_384',
+#     - 'sha3_512',
+#     - 'sha512',
+#     - 'shake_128',
+#     - 'shake_256'
+SUPPORTED_ALGORITHMS: FrozenSet[str] = frozenset({"CRC-64-AVRO"} | hashlib.algorithms_guaranteed)
+
+
+def _crc_64_fingerprint(data: bytes) -> bytes:
+    """The 64-bit Rabin Fingerprint.
+
+    As described in the Avro specification.
+
+    Args:
+        data: A bytes object containing the UTF-8 encoded parsing canonical
+        form of an Avro schema.
+    Returns:
+        A bytes object with a length of eight in little-endian format.
+    """
+    result = _EMPTY64_FINGERPRINT
+
+    for b in data:
+        result = (result >> 8) ^ _FINGERPRINT_TABLE[(result ^ b) & 0xFF]
+
+    # Although not mentioned in the Avro specification, the Java
+    # implementation gives fingerprint bytes in little-endian order
+    return result.to_bytes(length=8, byteorder="little", signed=False)
+
+
 #
 # Base Classes
 #
@@ -142,9 +197,11 @@ class PropertiesMixin:
         return get_other_props(self.props, self._reserved_properties)
 
 
-class EqualByJsonMixin:
+class EqualByJsonMixin(collections.abc.Hashable):
     """A mixin that defines equality as equal if the json deserializations are equal."""
 
+    fingerprint: Callable[..., bytes]
+
     def __eq__(self, that: object) -> bool:
         try:
             that_obj = json.loads(str(that))
@@ -152,13 +209,29 @@ class EqualByJsonMixin:
             return False
         return cast(bool, json.loads(str(self)) == that_obj)
 
+    def __hash__(self) -> int:
+        """Make it so a schema can be in a set or a key in a dictionary.
 
-class EqualByPropsMixin(PropertiesMixin):
+        NB: Python has special rules for this method being defined in the same class as __eq__.
+        """
+        return hash(self.fingerprint())
+
+
+class EqualByPropsMixin(collections.abc.Hashable, PropertiesMixin):
     """A mixin that defines equality as equal if the props are equal."""
 
+    fingerprint: Callable[..., bytes]
+
     def __eq__(self, that: object) -> bool:
         return hasattr(that, "props") and self.props == getattr(that, "props")
 
+    def __hash__(self) -> int:
+        """Make it so a schema can be in a set or a key in a dictionary.
+
+        NB: Python has special rules for this method being defined in the same class as __eq__.
+        """
+        return hash(self.fingerprint())
+
 
 class CanonicalPropertiesMixin(PropertiesMixin):
     """A Mixin that provides canonical properties to Schema and Field types."""
@@ -174,14 +247,15 @@ class Schema(abc.ABC, CanonicalPropertiesMixin):
 
     _reserved_properties = SCHEMA_RESERVED_PROPS
 
-    def __init__(self, type_: str, other_props: Optional[Mapping[str, object]] = None) -> None:
+    def __init__(self, type_: str, other_props: Optional[Mapping[str, object]] = None, validate_names: bool = True) -> None:
         if not isinstance(type_, str):
             raise avro.errors.SchemaParseException("Schema type must be a string.")
-        if type_ not in VALID_TYPES:
+        if type_ not in avro.constants.VALID_TYPES:
             raise avro.errors.SchemaParseException(f"{type_} is not a valid type.")
         self.set_prop("type", type_)
         self.type = type_
         self.props.update(other_props or {})
+        self.validate_names = validate_names
 
     @abc.abstractmethod
     def match(self, writer: "Schema") -> bool:
@@ -239,6 +313,30 @@ class Schema(abc.ABC, CanonicalPropertiesMixin):
         Consider the mixins EqualByPropsMixin and EqualByJsonMixin
         """
 
+    def fingerprint(self, algorithm="CRC-64-AVRO") -> bytes:
+        """
+        Generate fingerprint for supplied algorithm.
+
+        'CRC-64-AVRO' will be used as the algorithm by default, but any
+        algorithm supported by hashlib (as can be referenced with
+        `hashlib.algorithms_guaranteed`) can be specified.
+
+        `algorithm` param is used as an algorithm name, and NoSuchAlgorithmException
+        will be thrown if the algorithm is not among supported.
+        """
+        schema = self.canonical_form.encode("utf-8")
+
+        if algorithm == "CRC-64-AVRO":
+            return _crc_64_fingerprint(schema)
+
+        if algorithm not in SUPPORTED_ALGORITHMS:
+            raise avro.errors.UnknownFingerprintAlgorithmException(f"Unknown Fingerprint Algorithm: {algorithm}")
+
+        # Generate digests with hashlib for all other algorithms
+        # Lowercase algorithm to support algorithm strings sent by other languages like Java
+        h = hashlib.new(algorithm.lower(), schema)
+        return h.digest()
+
 
 class NamedSchema(Schema):
     """Named Schemas specified in NAMED_TYPES."""
@@ -250,8 +348,9 @@ class NamedSchema(Schema):
         namespace: Optional[str] = None,
         names: Optional[Names] = None,
         other_props: Optional[Mapping[str, object]] = None,
+        validate_names: bool = True,
     ) -> None:
-        super().__init__(type_, other_props)
+        super().__init__(type_, other_props, validate_names=validate_names)
         if not name:
             raise avro.errors.SchemaParseException("Named Schemas must have a non-empty name.")
         if not isinstance(name, str):
@@ -259,12 +358,12 @@ class NamedSchema(Schema):
         if namespace is not None and not isinstance(namespace, str):
             raise avro.errors.SchemaParseException("The namespace property must be a string.")
         namespace = namespace or None  # Empty string -> None
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
         new_name = names.add_name(name, namespace, self)
 
         # Store name and namespace as they were read in origin schema
-        self.set_prop("name", name)
-        if namespace is not None:
+        self.set_prop("name", new_name.name)
+        if new_name.space:
             self.set_prop("namespace", new_name.space)
 
         # Store full name as calculated from name, namespace
@@ -274,9 +373,17 @@ class NamedSchema(Schema):
         return self.name if self.namespace == names.default_namespace else self.fullname
 
     # read-only properties
-    name = property(lambda self: self.get_prop("name"))
-    namespace = property(lambda self: self.get_prop("namespace"))
-    fullname = property(lambda self: self._fullname)
+    @property
+    def name(self):
+        return self.get_prop("name")
+
+    @property
+    def namespace(self):
+        return self.get_prop("namespace")
+
+    @property
+    def fullname(self):
+        return self._fullname
 
 
 #
@@ -314,17 +421,7 @@ class DecimalLogicalSchema(LogicalSchema):
 class Field(CanonicalPropertiesMixin, EqualByJsonMixin):
     _reserved_properties: Sequence[str] = FIELD_RESERVED_PROPS
 
-    def __init__(
-        self,
-        type_,
-        name,
-        has_default,
-        default=None,
-        order=None,
-        names=None,
-        doc=None,
-        other_props=None,
-    ):
+    def __init__(self, type_, name, has_default, default=None, order=None, names=None, doc=None, other_props=None, validate_names: bool = True):
         if not name:
             raise avro.errors.SchemaParseException("Fields must have a non-empty name.")
         if not isinstance(name, str):
@@ -338,13 +435,14 @@ class Field(CanonicalPropertiesMixin, EqualByJsonMixin):
             type_schema = names.get_name(type_, None)
         else:
             try:
-                type_schema = make_avsc_object(type_, names)
+                type_schema = make_avsc_object(type_, names, validate_names=validate_names)
             except Exception as e:
                 raise avro.errors.SchemaParseException(f'Type property "{type_}" not a valid Avro schema: {e}')
         self.set_prop("type", type_schema)
         self.set_prop("name", name)
         self.type = type_schema
         self.name = name
+        self.validate_names = validate_names
         # TODO(hammer): check to ensure default is valid
         if has_default:
             self.set_prop("default", default)
@@ -354,16 +452,27 @@ class Field(CanonicalPropertiesMixin, EqualByJsonMixin):
             self.set_prop("doc", doc)
 
     # read-only properties
-    default = property(lambda self: self.get_prop("default"))
-    has_default = property(lambda self: self._has_default)
-    order = property(lambda self: self.get_prop("order"))
-    doc = property(lambda self: self.get_prop("doc"))
+    @property
+    def default(self):
+        return self.get_prop("default")
+
+    @property
+    def has_default(self):
+        return self._has_default
+
+    @property
+    def order(self):
+        return self.get_prop("order")
+
+    @property
+    def doc(self):
+        return self.get_prop("doc")
 
     def __str__(self):
         return json.dumps(self.to_json())
 
     def to_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         to_dump = self.props.copy()
         to_dump["type"] = self.type.to_json(names)
@@ -371,7 +480,7 @@ class Field(CanonicalPropertiesMixin, EqualByJsonMixin):
         return to_dump
 
     def to_canonical_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         to_dump = self.canonical_properties
         to_dump["type"] = self.type.to_canonical_json(names)
@@ -400,7 +509,7 @@ class PrimitiveSchema(EqualByPropsMixin, Schema):
 
     def __init__(self, type, other_props=None):
         # Ensure valid ctor args
-        if type not in PRIMITIVE_TYPES:
+        if type not in avro.constants.PRIMITIVE_TYPES:
             raise avro.errors.AvroException(f"{type} is not a valid primitive type.")
 
         # Call parent ctor
@@ -456,8 +565,13 @@ class BytesDecimalSchema(PrimitiveSchema, DecimalLogicalSchema):
         self.set_prop("scale", scale)
 
     # read-only properties
-    precision = property(lambda self: self.get_prop("precision"))
-    scale = property(lambda self: self.get_prop("scale"))
+    @property
+    def precision(self):
+        return self.get_prop("precision")
+
+    @property
+    def scale(self):
+        return self.get_prop("scale")
 
     def to_json(self, names=None):
         return self.props
@@ -471,20 +585,22 @@ class BytesDecimalSchema(PrimitiveSchema, DecimalLogicalSchema):
 # Complex Types (non-recursive)
 #
 class FixedSchema(EqualByPropsMixin, NamedSchema):
-    def __init__(self, name, namespace, size, names=None, other_props=None):
+    def __init__(self, name, namespace, size, names=None, other_props=None, validate_names: bool = True):
         # Ensure valid ctor args
         if not isinstance(size, int) or size < 0:
             fail_msg = "Fixed Schema requires a valid positive integer for size property."
             raise avro.errors.AvroException(fail_msg)
 
         # Call parent ctor
-        NamedSchema.__init__(self, "fixed", name, namespace, names, other_props)
+        NamedSchema.__init__(self, "fixed", name, namespace, names, other_props, validate_names=validate_names)
 
         # Add class members
         self.set_prop("size", size)
 
     # read-only properties
-    size = property(lambda self: self.get_prop("size"))
+    @property
+    def size(self):
+        return self.get_prop("size")
 
     def match(self, writer):
         """Return True if the current schema (as reader) matches the writer schema.
@@ -495,7 +611,7 @@ class FixedSchema(EqualByPropsMixin, NamedSchema):
         return self.type == writer.type and self.check_props(writer, ["fullname", "size"])
 
     def to_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         if self.fullname in names.names:
             return self.name_ref(names)
@@ -529,16 +645,22 @@ class FixedDecimalSchema(FixedSchema, DecimalLogicalSchema):
         namespace=None,
         names=None,
         other_props=None,
+        validate_names: bool = True,
     ):
         max_precision = int(math.floor(math.log10(2) * (8 * size - 1)))
         DecimalLogicalSchema.__init__(self, precision, scale, max_precision)
-        FixedSchema.__init__(self, name, namespace, size, names, other_props)
+        FixedSchema.__init__(self, name, namespace, size, names, other_props, validate_names=validate_names)
         self.set_prop("precision", precision)
         self.set_prop("scale", scale)
 
     # read-only properties
-    precision = property(lambda self: self.get_prop("precision"))
-    scale = property(lambda self: self.get_prop("scale"))
+    @property
+    def precision(self):
+        return self.get_prop("precision")
+
+    @property
+    def scale(self):
+        return self.get_prop("scale")
 
     def to_json(self, names=None):
         return self.props
@@ -558,9 +680,10 @@ class EnumSchema(EqualByPropsMixin, NamedSchema):
         doc: Optional[str] = None,
         other_props: Optional[Mapping[str, object]] = None,
         validate_enum_symbols: bool = True,
+        validate_names: bool = True,
     ) -> None:
         """
-        @arg validate_enum_symbols: If False, will allow enum symbols that are not valid Avro names.
+        @arg validate_enum_symbols: If False, will allow enum symbols that are not valid Avro names and default, which is not an enumerated symbol.
         """
         if validate_enum_symbols:
             for symbol in symbols:
@@ -573,14 +696,14 @@ class EnumSchema(EqualByPropsMixin, NamedSchema):
             raise avro.errors.AvroException(f"Duplicate symbol: {symbols}")
 
         # Call parent ctor
-        NamedSchema.__init__(self, "enum", name, namespace, names, other_props)
+        NamedSchema.__init__(self, "enum", name, namespace, names, other_props, validate_names)
 
         # Add class members
         self.set_prop("symbols", symbols)
         if doc is not None:
             self.set_prop("doc", doc)
 
-        if other_props and "default" in other_props:
+        if validate_enum_symbols and other_props and "default" in other_props:
             default = other_props["default"]
             if default not in symbols:
                 raise avro.errors.InvalidDefault(f"Enum default '{default}' is not a valid member of symbols '{symbols}'")
@@ -592,7 +715,9 @@ class EnumSchema(EqualByPropsMixin, NamedSchema):
             return symbols
         raise Exception
 
-    doc = property(lambda self: self.get_prop("doc"))
+    @property
+    def doc(self):
+        return self.get_prop("doc")
 
     def match(self, writer):
         """Return True if the current schema (as reader) matches the writer schema.
@@ -603,7 +728,7 @@ class EnumSchema(EqualByPropsMixin, NamedSchema):
         return self.type == writer.type and self.check_props(writer, ["fullname"])
 
     def to_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         if self.fullname in names.names:
             return self.name_ref(names)
@@ -633,16 +758,16 @@ class EnumSchema(EqualByPropsMixin, NamedSchema):
 
 
 class ArraySchema(EqualByJsonMixin, Schema):
-    def __init__(self, items, names=None, other_props=None):
+    def __init__(self, items, names=None, other_props=None, validate_names: bool = True):
         # Call parent ctor
-        Schema.__init__(self, "array", other_props)
+        Schema.__init__(self, "array", other_props, validate_names=validate_names)
         # Add class members
 
         if isinstance(items, str) and names.has_name(items, None):
             items_schema = names.get_name(items, None)
         else:
             try:
-                items_schema = make_avsc_object(items, names)
+                items_schema = make_avsc_object(items, names, validate_names=self.validate_names)
             except avro.errors.SchemaParseException as e:
                 fail_msg = f"Items schema ({items}) not a valid Avro schema: {e} (known names: {names.names.keys()})"
                 raise avro.errors.SchemaParseException(fail_msg)
@@ -650,7 +775,9 @@ class ArraySchema(EqualByJsonMixin, Schema):
         self.set_prop("items", items_schema)
 
     # read-only properties
-    items = property(lambda self: self.get_prop("items"))
+    @property
+    def items(self):
+        return self.get_prop("items")
 
     def match(self, writer):
         """Return True if the current schema (as reader) matches the writer schema.
@@ -661,7 +788,7 @@ class ArraySchema(EqualByJsonMixin, Schema):
         return self.type == writer.type and self.items.check_props(writer.items, ["type"])
 
     def to_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         to_dump = self.props.copy()
         item_schema = self.get_prop("items")
@@ -670,7 +797,7 @@ class ArraySchema(EqualByJsonMixin, Schema):
         return to_dump
 
     def to_canonical_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         to_dump = self.canonical_properties
         item_schema = self.get_prop("items")
@@ -684,16 +811,16 @@ class ArraySchema(EqualByJsonMixin, Schema):
 
 
 class MapSchema(EqualByJsonMixin, Schema):
-    def __init__(self, values, names=None, other_props=None):
+    def __init__(self, values, names=None, other_props=None, validate_names: bool = True):
         # Call parent ctor
-        Schema.__init__(self, "map", other_props)
+        Schema.__init__(self, "map", other_props, validate_names=validate_names)
 
         # Add class members
         if isinstance(values, str) and names.has_name(values, None):
             values_schema = names.get_name(values, None)
         else:
             try:
-                values_schema = make_avsc_object(values, names)
+                values_schema = make_avsc_object(values, names, validate_names=self.validate_names)
             except avro.errors.SchemaParseException:
                 raise
             except Exception:
@@ -702,7 +829,9 @@ class MapSchema(EqualByJsonMixin, Schema):
         self.set_prop("values", values_schema)
 
     # read-only properties
-    values = property(lambda self: self.get_prop("values"))
+    @property
+    def values(self):
+        return self.get_prop("values")
 
     def match(self, writer):
         """Return True if the current schema (as reader) matches the writer schema.
@@ -713,7 +842,7 @@ class MapSchema(EqualByJsonMixin, Schema):
         return writer.type == self.type and self.values.check_props(writer.values, ["type"])
 
     def to_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         to_dump = self.props.copy()
         to_dump["values"] = self.get_prop("values").to_json(names)
@@ -721,7 +850,7 @@ class MapSchema(EqualByJsonMixin, Schema):
         return to_dump
 
     def to_canonical_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         to_dump = self.canonical_properties
         to_dump["values"] = self.get_prop("values").to_canonical_json(names)
@@ -738,29 +867,29 @@ class UnionSchema(EqualByJsonMixin, Schema):
     names is a dictionary of schema objects
     """
 
-    def __init__(self, schemas, names=None):
+    def __init__(self, schemas, names=None, validate_names: bool = True):
         # Ensure valid ctor args
         if not isinstance(schemas, list):
             fail_msg = "Union schema requires a list of schemas."
             raise avro.errors.SchemaParseException(fail_msg)
 
         # Call parent ctor
-        Schema.__init__(self, "union")
+        Schema.__init__(self, "union", validate_names=validate_names)
 
         # Add class members
-        schema_objects = []
+        schema_objects: List[Schema] = []
         for schema in schemas:
             if isinstance(schema, str) and names.has_name(schema, None):
                 new_schema = names.get_name(schema, None)
             else:
                 try:
-                    new_schema = make_avsc_object(schema, names)
+                    new_schema = make_avsc_object(schema, names, validate_names=self.validate_names)
                 except Exception as e:
                     raise avro.errors.SchemaParseException(f"Union item must be a valid Avro schema: {e}")
             # check the new schema
             if (
-                new_schema.type in VALID_TYPES
-                and new_schema.type not in NAMED_TYPES
+                new_schema.type in avro.constants.VALID_TYPES
+                and new_schema.type not in avro.constants.NAMED_TYPES
                 and new_schema.type in [schema.type for schema in schema_objects]
             ):
                 raise avro.errors.SchemaParseException(f"{new_schema.type} type already in Union")
@@ -771,7 +900,9 @@ class UnionSchema(EqualByJsonMixin, Schema):
         self._schemas = schema_objects
 
     # read-only properties
-    schemas = property(lambda self: self._schemas)
+    @property
+    def schemas(self):
+        return self._schemas
 
     def match(self, writer):
         """Return True if the current schema (as reader) matches the writer schema.
@@ -782,7 +913,7 @@ class UnionSchema(EqualByJsonMixin, Schema):
         return writer.type in {"union", "error_union"} or any(s.match(writer) for s in self.schemas)
 
     def to_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         to_dump = []
         for schema in self.schemas:
@@ -791,24 +922,22 @@ class UnionSchema(EqualByJsonMixin, Schema):
         return to_dump
 
     def to_canonical_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         return [schema.to_canonical_json(names) for schema in self.schemas]
 
     def validate(self, datum):
         """Return the first branch schema of which datum is a valid example, else None."""
-        for branch in self.schemas:
-            if branch.validate(datum) is not None:
-                return branch
+        return next((branch for branch in self.schemas if branch.validate(datum) is not None), None)
 
 
 class ErrorUnionSchema(UnionSchema):
-    def __init__(self, schemas, names=None):
+    def __init__(self, schemas, names=None, validate_names: bool = True):
         # Prepend "string" to handle system errors
-        UnionSchema.__init__(self, ["string"] + schemas, names)
+        UnionSchema.__init__(self, ["string"] + schemas, names, validate_names)
 
     def to_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         to_dump = []
         for schema in self.schemas:
@@ -822,7 +951,7 @@ class ErrorUnionSchema(UnionSchema):
 
 class RecordSchema(EqualByJsonMixin, NamedSchema):
     @staticmethod
-    def make_field_objects(field_data: Sequence[Mapping[str, object]], names: avro.name.Names) -> Sequence[Field]:
+    def make_field_objects(field_data: Sequence[Mapping[str, object]], names: avro.name.Names, validate_names: bool = True) -> Sequence[Field]:
         """We're going to need to make message parameters too."""
         field_objects = []
         field_names = []
@@ -838,7 +967,7 @@ class RecordSchema(EqualByJsonMixin, NamedSchema):
             order = field.get("order")
             doc = field.get("doc")
             other_props = get_other_props(field, FIELD_RESERVED_PROPS)
-            new_field = Field(type, name, has_default, default, order, names, doc, other_props)
+            new_field = Field(type, name, has_default, default, order, names, doc, other_props, validate_names=validate_names)
             # make sure field name has not been used yet
             if new_field.name in field_names:
                 fail_msg = f"Field name {new_field.name} already in use."
@@ -864,6 +993,7 @@ class RecordSchema(EqualByJsonMixin, NamedSchema):
         schema_type="record",
         doc=None,
         other_props=None,
+        validate_names: bool = True,
     ):
         # Ensure valid ctor args
         if fields is None:
@@ -877,14 +1007,15 @@ class RecordSchema(EqualByJsonMixin, NamedSchema):
         if schema_type == "request":
             Schema.__init__(self, schema_type, other_props)
         else:
-            NamedSchema.__init__(self, schema_type, name, namespace, names, other_props)
+            NamedSchema.__init__(self, schema_type, name, namespace, names, other_props, validate_names=validate_names)
 
+        names = names or Names(validate_names=self.validate_names)
         if schema_type == "record":
             old_default = names.default_namespace
-            names.default_namespace = Name(name, namespace, names.default_namespace).space
+            names.default_namespace = Name(name, namespace, names.default_namespace, validate_name=validate_names).space
 
         # Add class members
-        field_objects = RecordSchema.make_field_objects(fields, names)
+        field_objects = RecordSchema.make_field_objects(fields, names, validate_names=validate_names)
         self.set_prop("fields", field_objects)
         if doc is not None:
             self.set_prop("doc", doc)
@@ -893,8 +1024,13 @@ class RecordSchema(EqualByJsonMixin, NamedSchema):
             names.default_namespace = old_default
 
     # read-only properties
-    fields = property(lambda self: self.get_prop("fields"))
-    doc = property(lambda self: self.get_prop("doc"))
+    @property
+    def fields(self):
+        return self.get_prop("fields")
+
+    @property
+    def doc(self):
+        return self.get_prop("doc")
 
     @property
     def fields_dict(self):
@@ -904,7 +1040,7 @@ class RecordSchema(EqualByJsonMixin, NamedSchema):
         return fields_dict
 
     def to_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         # Request records don't have names
         if self.type == "request":
@@ -921,7 +1057,7 @@ class RecordSchema(EqualByJsonMixin, NamedSchema):
         return to_dump
 
     def to_canonical_json(self, names=None):
-        names = names or Names()
+        names = names or Names(validate_names=self.validate_names)
 
         if self.type == "request":
             raise NotImplementedError("Canonical form (probably) does not make sense on type request")
@@ -1045,10 +1181,8 @@ class UUIDSchema(LogicalSchema, PrimitiveSchema):
 
     def validate(self, datum):
         try:
-            val = uuid.UUID(datum)
-        except ValueError:
-            # If it's a value error, then the string
-            # is not a valid hex code for a UUID.
+            uuid.UUID(datum)
+        except (ValueError, TypeError):
             return None
 
         return self
@@ -1102,14 +1236,16 @@ def make_logical_schema(logical_type, type_, other_props):
     return None
 
 
-def make_avsc_object(json_data: object, names: Optional[avro.name.Names] = None, validate_enum_symbols: bool = True) -> Schema:
+def make_avsc_object(
+    json_data: object, names: Optional[avro.name.Names] = None, validate_enum_symbols: bool = True, validate_names: bool = True
+) -> Schema:
     """
     Build Avro Schema from data parsed out of JSON string.
 
     @arg names: A Names object (tracks seen names and default space)
     @arg validate_enum_symbols: If False, will allow enum symbols that are not valid Avro names.
     """
-    names = names or Names()
+    names = names or Names(validate_names=validate_names)
 
     # JSON object (non-union)
     if callable(getattr(json_data, "get", None)):
@@ -1123,7 +1259,7 @@ def make_avsc_object(json_data: object, names: Optional[avro.name.Names] = None,
             if logical_schema is not None:
                 return cast(Schema, logical_schema)
 
-        if type_ in NAMED_TYPES:
+        if type_ in avro.constants.NAMED_TYPES:
             name = json_data.get("name")
             if not isinstance(name, str):
                 raise avro.errors.SchemaParseException(f"Name {name} must be a string, but it is {type(name)}.")
@@ -1134,10 +1270,10 @@ def make_avsc_object(json_data: object, names: Optional[avro.name.Names] = None,
                     precision = json_data.get("precision")
                     scale = json_data.get("scale", 0)
                     try:
-                        return FixedDecimalSchema(size, name, precision, scale, namespace, names, other_props)
+                        return FixedDecimalSchema(size, name, precision, scale, namespace, names, other_props, validate_names)
                     except avro.errors.IgnoredLogicalType as warning:
                         warnings.warn(warning)
-                return FixedSchema(name, namespace, size, names, other_props)
+                return FixedSchema(name, namespace, size, names, other_props, validate_names=validate_names)
             elif type_ == "enum":
                 symbols = json_data.get("symbols")
                 if not isinstance(symbols, Sequence):
@@ -1146,34 +1282,26 @@ def make_avsc_object(json_data: object, names: Optional[avro.name.Names] = None,
                     if not isinstance(symbol, str):
                         raise avro.errors.SchemaParseException(f"Enum symbols must be a sequence of strings, but one symbol is a {type(symbol)}")
                 doc = json_data.get("doc")
-                return EnumSchema(
-                    name,
-                    namespace,
-                    symbols,
-                    names,
-                    doc,
-                    other_props,
-                    validate_enum_symbols,
-                )
+                return EnumSchema(name, namespace, symbols, names, doc, other_props, validate_enum_symbols, validate_names)
             if type_ in ["record", "error"]:
                 fields = json_data.get("fields")
                 doc = json_data.get("doc")
-                return RecordSchema(name, namespace, fields, names, type_, doc, other_props)
+                return RecordSchema(name, namespace, fields, names, type_, doc, other_props, validate_names)
             raise avro.errors.SchemaParseException(f"Unknown Named Type: {type_}")
 
-        if type_ in PRIMITIVE_TYPES:
+        if type_ in avro.constants.PRIMITIVE_TYPES:
             return PrimitiveSchema(type_, other_props)
 
-        if type_ in VALID_TYPES:
+        if type_ in avro.constants.VALID_TYPES:
             if type_ == "array":
                 items = json_data.get("items")
-                return ArraySchema(items, names, other_props)
+                return ArraySchema(items, names, other_props, validate_names)
             elif type_ == "map":
                 values = json_data.get("values")
-                return MapSchema(values, names, other_props)
+                return MapSchema(values, names, other_props, validate_names)
             elif type_ == "error_union":
                 declared_errors = json_data.get("declared_errors")
-                return ErrorUnionSchema(declared_errors, names)
+                return ErrorUnionSchema(declared_errors, names, validate_names)
             else:
                 raise avro.errors.SchemaParseException(f"Unknown Valid Type: {type_}")
         elif type_ is None:
@@ -1182,31 +1310,33 @@ def make_avsc_object(json_data: object, names: Optional[avro.name.Names] = None,
             raise avro.errors.SchemaParseException(f"Undefined type: {type_}")
     # JSON array (union)
     elif isinstance(json_data, list):
-        return UnionSchema(json_data, names)
+        return UnionSchema(json_data, names, validate_names=validate_names)
     # JSON string (primitive)
-    elif json_data in PRIMITIVE_TYPES:
+    elif json_data in avro.constants.PRIMITIVE_TYPES:
         return PrimitiveSchema(json_data)
     # not for us!
     fail_msg = f"Could not make an Avro Schema object from {json_data}"
     raise avro.errors.SchemaParseException(fail_msg)
 
 
-def parse(json_string: str, validate_enum_symbols: bool = True) -> Schema:
+def parse(json_string: str, validate_enum_symbols: bool = True, validate_names: bool = True) -> Schema:
     """Constructs the Schema from the JSON text.
 
     @arg json_string: The json string of the schema to parse
     @arg validate_enum_symbols: If False, will allow enum symbols that are not valid Avro names.
+    @arg validate_names: If False, will allow names that are not valid Avro names. When disabling the validation
+                         test the non-compliant names for non-compliant behavior, also in interoperability cases.
     @return Schema
     """
     try:
         json_data = json.loads(json_string)
     except json.decoder.JSONDecodeError as e:
         raise avro.errors.SchemaParseException(f"Error parsing JSON: {json_string}, error = {e}") from e
-    return make_avsc_object(json_data, Names(), validate_enum_symbols)
+    return make_avsc_object(json_data, Names(validate_names=validate_names), validate_enum_symbols, validate_names)
 
 
-def from_path(path: Union[Path, str], validate_enum_symbols: bool = True) -> Schema:
+def from_path(path: Union[Path, str], validate_enum_symbols: bool = True, validate_names: bool = True) -> Schema:
     """
     Constructs the Schema from a path to an avsc (json) file.
     """
-    return parse(Path(path).read_text(), validate_enum_symbols)
+    return parse(Path(path).read_text(), validate_enum_symbols, validate_names)
