@@ -58,12 +58,12 @@ class InvalidTestSchema(TestSchema):
     valid = False
 
 
-PRIMITIVE_EXAMPLES = [InvalidTestSchema('"True"')]  # type: List[TestSchema]
+PRIMITIVE_EXAMPLES: List[TestSchema] = [InvalidTestSchema('"True"')]
 PRIMITIVE_EXAMPLES.append(InvalidTestSchema("True"))
 PRIMITIVE_EXAMPLES.append(InvalidTestSchema('{"no_type": "test"}'))
 PRIMITIVE_EXAMPLES.append(InvalidTestSchema('{"type": "panther"}'))
-PRIMITIVE_EXAMPLES.extend([ValidTestSchema(f'"{t}"') for t in avro.schema.PRIMITIVE_TYPES])
-PRIMITIVE_EXAMPLES.extend([ValidTestSchema({"type": t}) for t in avro.schema.PRIMITIVE_TYPES])
+PRIMITIVE_EXAMPLES.extend([ValidTestSchema(f'"{t}"') for t in avro.constants.PRIMITIVE_TYPES])
+PRIMITIVE_EXAMPLES.extend([ValidTestSchema({"type": t}) for t in avro.constants.PRIMITIVE_TYPES])
 
 FIXED_EXAMPLES = [
     ValidTestSchema({"type": "fixed", "name": "Test", "size": 1}),
@@ -519,6 +519,204 @@ IGNORED_LOGICAL_TYPE = [
     ),
 ]
 
+
+# Fingerprint examples are in the form of tuples:
+# - Value in Position 0 is schema
+# - Value in Position 1 is an array of fingerprints:
+#     - Position 0 is CRC-64-AVRO fingerprint
+#     - Position 0 is MD5 fingerprint
+#     - Position 0 is SHA256 fingerprint
+FINGERPRINT_EXAMPLES = [
+    ('"int"', ["8f5c393f1ad57572", "ef524ea1b91e73173d938ade36c1db32", "3f2b87a9fe7cc9b13835598c3981cd45e3e355309e5090aa0933d7becb6fba45"]),
+    ('{"type": "int"}', ["8f5c393f1ad57572", "ef524ea1b91e73173d938ade36c1db32", "3f2b87a9fe7cc9b13835598c3981cd45e3e355309e5090aa0933d7becb6fba45"]),
+    ('"float"', ["90d7a83ecb027c4d", "50a6b9db85da367a6d2df400a41758a6", "1e71f9ec051d663f56b0d8e1fc84d71aa56ccfe9fa93aa20d10547a7abeb5cc0"]),
+    (
+        '{"type": "float"}',
+        ["90d7a83ecb027c4d", "50a6b9db85da367a6d2df400a41758a6", "1e71f9ec051d663f56b0d8e1fc84d71aa56ccfe9fa93aa20d10547a7abeb5cc0"],
+    ),
+    ('"long"', ["b71df49344e154d0", "e1dd9a1ef98b451b53690370b393966b", "c32c497df6730c97fa07362aa5023f37d49a027ec452360778114cf427965add"]),
+    (
+        '{"type": "long"}',
+        ["b71df49344e154d0", "e1dd9a1ef98b451b53690370b393966b", "c32c497df6730c97fa07362aa5023f37d49a027ec452360778114cf427965add"],
+    ),
+    ('"double"', ["7e95ab32c035758e", "bfc71a62f38b99d6a93690deeb4b3af6", "730a9a8c611681d7eef442e03c16c70d13bca3eb8b977bb403eaff52176af254"]),
+    (
+        '{"type": "double"}',
+        ["7e95ab32c035758e", "bfc71a62f38b99d6a93690deeb4b3af6", "730a9a8c611681d7eef442e03c16c70d13bca3eb8b977bb403eaff52176af254"],
+    ),
+    ('"bytes"', ["651920c3da16c04f", "b462f06cb909be57c85008867784cde6", "9ae507a9dd39ee5b7c7e285da2c0846521c8ae8d80feeae5504e0c981d53f5fa"]),
+    (
+        '{"type": "bytes"}',
+        ["651920c3da16c04f", "b462f06cb909be57c85008867784cde6", "9ae507a9dd39ee5b7c7e285da2c0846521c8ae8d80feeae5504e0c981d53f5fa"],
+    ),
+    ('"string"', ["c70345637248018f", "095d71cf12556b9d5e330ad575b3df5d", "e9e5c1c9e4f6277339d1bcde0733a59bd42f8731f449da6dc13010a916930d48"]),
+    (
+        '{"type": "string"}',
+        ["c70345637248018f", "095d71cf12556b9d5e330ad575b3df5d", "e9e5c1c9e4f6277339d1bcde0733a59bd42f8731f449da6dc13010a916930d48"],
+    ),
+    ('"boolean"', ["64f7d4a478fc429f", "01f692b30d4a1c8a3e600b1440637f8f", "a5b031ab62bc416d720c0410d802ea46b910c4fbe85c50a946ccc658b74e677e"]),
+    (
+        '{"type": "boolean"}',
+        ["64f7d4a478fc429f", "01f692b30d4a1c8a3e600b1440637f8f", "a5b031ab62bc416d720c0410d802ea46b910c4fbe85c50a946ccc658b74e677e"],
+    ),
+    ('"null"', ["8a8f25cce724dd63", "9b41ef67651c18488a8b08bb67c75699", "f072cbec3bf8841871d4284230c5e983dc211a56837aed862487148f947d1a1f"]),
+    (
+        '{"type": "null"}',
+        ["8a8f25cce724dd63", "9b41ef67651c18488a8b08bb67c75699", "f072cbec3bf8841871d4284230c5e983dc211a56837aed862487148f947d1a1f"],
+    ),
+    (
+        '{"type": "fixed", "name": "Test", "size": 1}',
+        ["6869897b4049355b", "db01bc515fcfcd2d4be82ed385288261", "f527116a6f44455697e935afc31dc60ad0f95caf35e1d9c9db62edb3ffeb9170"],
+    ),
+    (
+        json.dumps({"type": "fixed", "name": "MyFixed", "namespace": "org.apache.hadoop.avro", "size": 1}),
+        ["fadbd138e85bdf45", "d74b3726484422711c465d49e857b1ba", "28e493a44771cecc5deca4bd938cdc3d5a24cfe1f3760bc938fa1057df6334fc"],
+    ),
+    (
+        '{"type": "enum", "name": "Test", "symbols": ["A", "B"]}',
+        ["03a2f2c2e27f7a16", "d883f2a9b16ed085fcc5e4ca6c8f6ed1", "9b51286144f87ce5aebdc61ca834379effa5a41ce6ac0938630ff246297caca8"],
+    ),
+    (
+        '{"type": "array", "items": "long"}',
+        ["715e2ea28bc91654", "c1c387e8d6a58f0df749b698991b1f43", "f78e954167feb23dcb1ce01e8463cebf3408e0a4259e16f24bd38f6d0f1d578b"],
+    ),
+    (
+        json.dumps({"type": "array", "items": {"type": "enum", "name": "Test", "symbols": ["A", "B"]}}),
+        ["10d9ade1fa3a0387", "cfc7b861c7cfef082a6ef082948893fa", "0d8edd49d7f7e9553668f133577bc99f842852b55d9f84f1f7511e4961aa685c"],
+    ),
+    (
+        '{"type": "map", "values": "long"}',
+        ["6f74f4e409b1334e", "32b3f1a3177a0e73017920f00448b56e", "b8fad07d458971a07692206b8a7cf626c86c62fe6bcff7c1b11bc7295de34853"],
+    ),
+    (
+        json.dumps({"type": "map", "values": {"type": "enum", "name": "Test", "symbols": ["A", "B"]}}),
+        ["df2ab0626f6b812d", "c588da6ba99701c41e73fd30d23f994e", "3886747ed1669a8af476b549e97b34222afb2fed5f18bb27c6f367ea0351a576"],
+    ),
+    (
+        '["string", "null", "long"]',
+        ["65a5be410d687566", "b11cf95f0a55dd55f9ee515a37bf937a", "ed8d254116441bb35e237ad0563cf5432b8c975334bd222c1ee84609435d95bb"],
+    ),
+    (
+        json.dumps({"type": "record", "name": "Test", "fields": [{"name": "f", "type": "long"}]}),
+        ["ed94e5f5e6eb588e", "69531a03db788afe353244cd049b1e6d", "9670f15a8f96d23e92830d00b8bd57275e02e3e173ffef7c253c170b6beabeb8"],
+    ),
+    (
+        json.dumps(
+            {
+                "type": "record",
+                "name": "Node",
+                "fields": [{"name": "label", "type": "string"}, {"name": "children", "type": {"type": "array", "items": "Node"}}],
+            }
+        ),
+        ["52cba544c3e756b7", "99625b0cc02050363e89ef66b0f406c9", "65d80dc8c95c98a9671d92cf0415edfabfee2cb058df2138606656cd6ae4dc59"],
+    ),
+    (
+        json.dumps(
+            {
+                "type": "record",
+                "name": "Lisp",
+                "fields": [
+                    {
+                        "name": "value",
+                        "type": [
+                            "null",
+                            "string",
+                            {"type": "record", "name": "Cons", "fields": [{"name": "car", "type": "Lisp"}, {"name": "cdr", "type": "Lisp"}]},
+                        ],
+                    }
+                ],
+            }
+        ),
+        ["68d91a23eda0b306", "9e1d0d15b52789fcb8e3a88b53059d5f", "e5ce4f4a15ce19fa1047cfe16a3b0e13a755db40f00f23284fdd376fc1c7dd21"],
+    ),
+    (
+        json.dumps(
+            {
+                "type": "record",
+                "name": "HandshakeRequest",
+                "namespace": "org.apache.avro.ipc",
+                "fields": [
+                    {"name": "clientHash", "type": {"type": "fixed", "name": "MD5", "size": 16}},
+                    {"name": "clientProtocol", "type": ["null", "string"]},
+                    {"name": "serverHash", "type": "MD5"},
+                    {"name": "meta", "type": ["null", {"type": "map", "values": "bytes"}]},
+                ],
+            }
+        ),
+        ["43818703b7b5d769", "16ded8b5027e80a17704c6565c0c3f1b", "6c317314687da52a85c813a7f0c92298a60b79625b9acc072e4d9e4256a1d800"],
+    ),
+    (
+        json.dumps(
+            {
+                "type": "record",
+                "name": "HandshakeResponse",
+                "namespace": "org.apache.avro.ipc",
+                "fields": [
+                    {"name": "match", "type": {"type": "enum", "name": "HandshakeMatch", "symbols": ["BOTH", "CLIENT", "NONE"]}},
+                    {"name": "serverProtocol", "type": ["null", "string"]},
+                    {"name": "serverHash", "type": ["null", {"name": "MD5", "size": 16, "type": "fixed"}]},
+                    {"name": "meta", "type": ["null", {"type": "map", "values": "bytes"}]},
+                ],
+            }
+        ),
+        ["00feee01de4ea50e", "afe529d01132daab7f4e2a6663e7a2f5", "a303cbbfe13958f880605d70c521a4b7be34d9265ac5a848f25916a67b11d889"],
+    ),
+    (
+        json.dumps(
+            {
+                "type": "record",
+                "name": "Interop",
+                "namespace": "org.apache.avro",
+                "fields": [
+                    {"name": "intField", "type": "int"},
+                    {"name": "longField", "type": "long"},
+                    {"name": "stringField", "type": "string"},
+                    {"name": "boolField", "type": "boolean"},
+                    {"name": "floatField", "type": "float"},
+                    {"name": "doubleField", "type": "double"},
+                    {"name": "bytesField", "type": "bytes"},
+                    {"name": "nullField", "type": "null"},
+                    {"name": "arrayField", "type": {"type": "array", "items": "double"}},
+                    {
+                        "name": "mapField",
+                        "type": {"type": "map", "values": {"name": "Foo", "type": "record", "fields": [{"name": "label", "type": "string"}]}},
+                    },
+                    {"name": "unionField", "type": ["boolean", "double", {"type": "array", "items": "bytes"}]},
+                    {"name": "enumField", "type": {"type": "enum", "name": "Kind", "symbols": ["A", "B", "C"]}},
+                    {"name": "fixedField", "type": {"type": "fixed", "name": "MD5", "size": 16}},
+                    {
+                        "name": "recordField",
+                        "type": {
+                            "type": "record",
+                            "name": "Node",
+                            "fields": [{"name": "label", "type": "string"}, {"name": "children", "type": {"type": "array", "items": "Node"}}],
+                        },
+                    },
+                ],
+            }
+        ),
+        ["e82c0a93a6a0b5a4", "994fea1a1be7ff8603cbe40c3bc7e4ca", "cccfd6e3f917cf53b0f90c206342e6703b0d905071f724a1c1f85b731c74058d"],
+    ),
+    (
+        json.dumps(
+            {
+                "type": "record",
+                "name": "ipAddr",
+                "fields": [{"name": "addr", "type": [{"name": "IPv6", "type": "fixed", "size": 16}, {"name": "IPv4", "type": "fixed", "size": 4}]}],
+            }
+        ),
+        ["8d961b4e298a1844", "45d85c69b353a99b93d7c4f2fcf0c30d", "6f6fc8f685a4f07d99734946565d63108806d55a8620febea047cf52cb0ac181"],
+    ),
+    (
+        json.dumps({"type": "record", "name": "TestDoc", "doc": "Doc string", "fields": [{"name": "name", "type": "string", "doc": "Doc String"}]}),
+        ["0e6660f02bcdc109", "f2da75f5131f5ab80629538287b8beb2", "0b3644f7aa5ca2fc4bad93ca2d3609c12aa9dbda9c15e68b34c120beff08e7b9"],
+    ),
+    (
+        '{"type": "enum", "name": "Test", "symbols": ["A", "B"], "doc": "Doc String"}',
+        ["03a2f2c2e27f7a16", "d883f2a9b16ed085fcc5e4ca6c8f6ed1", "9b51286144f87ce5aebdc61ca834379effa5a41ce6ac0938630ff246297caca8"],
+    ),
+]
+
 EXAMPLES = PRIMITIVE_EXAMPLES
 EXAMPLES += FIXED_EXAMPLES
 EXAMPLES += ENUM_EXAMPLES
@@ -564,78 +762,6 @@ class TestMisc(unittest.TestCase):
         # If we've made it this far, the subschema was reasonably stringified; it ccould be reparsed.
         self.assertEqual("X", t.fields[0].type.name)
 
-    def test_name_is_none(self):
-        """When a name is None its namespace is None."""
-        self.assertIsNone(avro.schema.Name(None, None, None).fullname)
-        self.assertIsNone(avro.schema.Name(None, None, None).space)
-
-    def test_name_not_empty_string(self):
-        """A name cannot be the empty string."""
-        self.assertRaises(avro.errors.SchemaParseException, avro.schema.Name, "", None, None)
-
-    def test_name_space_specified(self):
-        """Space combines with a name to become the fullname."""
-        # name and namespace specified
-        fullname = avro.schema.Name("a", "o.a.h", None).fullname
-        self.assertEqual(fullname, "o.a.h.a")
-
-    def test_fullname_space_specified(self):
-        """When name contains dots, namespace should be ignored."""
-        fullname = avro.schema.Name("a.b.c.d", "o.a.h", None).fullname
-        self.assertEqual(fullname, "a.b.c.d")
-
-    def test_name_default_specified(self):
-        """Default space becomes the namespace when the namespace is None."""
-        fullname = avro.schema.Name("a", None, "b.c.d").fullname
-        self.assertEqual(fullname, "b.c.d.a")
-
-    def test_fullname_default_specified(self):
-        """When a name contains dots, default space should be ignored."""
-        fullname = avro.schema.Name("a.b.c.d", None, "o.a.h").fullname
-        self.assertEqual(fullname, "a.b.c.d")
-
-    def test_fullname_space_default_specified(self):
-        """When a name contains dots, namespace and default space should be ignored."""
-        fullname = avro.schema.Name("a.b.c.d", "o.a.a", "o.a.h").fullname
-        self.assertEqual(fullname, "a.b.c.d")
-
-    def test_name_space_default_specified(self):
-        """When name and space are specified, default space should be ignored."""
-        fullname = avro.schema.Name("a", "o.a.a", "o.a.h").fullname
-        self.assertEqual(fullname, "o.a.a.a")
-
-    def test_equal_names(self):
-        """Equality of names is defined on the fullname and is case-sensitive."""
-        self.assertEqual(
-            avro.schema.Name("a.b.c.d", None, None),
-            avro.schema.Name("d", "a.b.c", None),
-        )
-        self.assertNotEqual(avro.schema.Name("C.d", None, None), avro.schema.Name("c.d", None, None))
-
-    def test_invalid_name(self):
-        """The name portion of a fullname, record field names, and enum symbols must:
-        start with [A-Za-z_] and subsequently contain only [A-Za-z0-9_]"""
-        self.assertRaises(
-            avro.errors.InvalidName,
-            avro.schema.Name,
-            "an especially spacey cowboy",
-            None,
-            None,
-        )
-        self.assertRaises(
-            avro.errors.InvalidName,
-            avro.schema.Name,
-            "99 problems but a name aint one",
-            None,
-            None,
-        )
-
-    def test_null_namespace(self):
-        """The empty string may be used as a namespace to indicate the null namespace."""
-        name = avro.schema.Name("name", "", None)
-        self.assertEqual(name.fullname, "name")
-        self.assertIsNone(name.space)
-
     def test_exception_is_not_swallowed_on_parse_error(self):
         """A specific exception message should appear on a json parse error."""
         self.assertRaisesRegex(
@@ -706,13 +832,25 @@ class TestMisc(unittest.TestCase):
     def test_parse_invalid_symbol(self):
         """Disabling enumschema symbol validation should allow invalid symbols to pass."""
         test_schema_string = json.dumps({"type": "enum", "name": "AVRO2174", "symbols": ["white space"]})
+
         with self.assertRaises(avro.errors.InvalidName, msg="When enum symbol validation is enabled, an invalid symbol should raise InvalidName."):
             avro.schema.parse(test_schema_string, validate_enum_symbols=True)
+
         try:
             avro.schema.parse(test_schema_string, validate_enum_symbols=False)
         except avro.errors.InvalidName:  # pragma: no coverage
             self.fail("When enum symbol validation is disabled, an invalid symbol should not raise InvalidName.")
 
+    def test_unsupported_fingerprint_algorithm(self):
+        s = avro.schema.parse('"int"')
+        self.assertRaises(avro.errors.UnknownFingerprintAlgorithmException, s.fingerprint, "foo")
+
+    def test_less_popular_fingerprint_algorithm(self):
+        s = avro.schema.parse('"int"')
+        fingerprint = s.fingerprint("sha384")
+        hex_fingerprint = "".join(format(b, "02x") for b in fingerprint).zfill(16)
+        self.assertEqual(hex_fingerprint, "32ed5e4ac896570f044d1dab68f4c8ca9866ac06d22261f399316bf4799e16854750238085775107dfac905c82b2feaf")
+
 
 class SchemaParseTestCase(unittest.TestCase):
     """Enable generating parse test cases over all the valid and invalid example schema."""
@@ -734,7 +872,7 @@ class SchemaParseTestCase(unittest.TestCase):
         try:
             warnings.filterwarnings(action="error", category=avro.errors.IgnoredLogicalType)
             self.test_schema.parse()
-        except (avro.errors.IgnoredLogicalType) as e:
+        except avro.errors.IgnoredLogicalType as e:
             self.assertIn(type(e), (type(w) for w in test_warnings))
             self.assertIn(str(e), (str(w) for w in test_warnings))
         except (avro.errors.AvroException, avro.errors.SchemaParseException):  # pragma: no coverage
@@ -752,6 +890,31 @@ class SchemaParseTestCase(unittest.TestCase):
             self.test_schema.parse()
 
 
+class HashableTestCase(unittest.TestCase):
+    """Ensure that Schema are hashable.
+
+    While hashability is implemented with parsing canonical form fingerprinting,
+    this test should be kept distinct to avoid coupling."""
+
+    def __init__(self, test_schema):
+        """Ignore the normal signature for unittest.TestCase because we are generating
+        many test cases from this one class. This is safe as long as the autoloader
+        ignores this class. The autoloader will ignore this class as long as it has
+        no methods starting with `test_`.
+        """
+        super().__init__("parse_and_hash")
+        self.test_schema = test_schema
+
+    def parse_and_hash(self):
+        """Ensure that every schema can be hashed."""
+        try:
+            hash(self.test_schema.parse())
+        except TypeError as e:
+            if "unhashable type" in str(e):
+                self.fail(f"{self.test_schema} is not hashable")
+            raise
+
+
 class RoundTripParseTestCase(unittest.TestCase):
     """Enable generating round-trip parse test cases over all the valid test schema."""
 
@@ -1253,6 +1416,39 @@ class CanonicalFormTestCase(unittest.TestCase):
         )
 
 
+class FingerprintTestCase(unittest.TestCase):
+    """
+    Enable generating fingerprint test cases across algorithms.
+
+    Fingerprint examples are in the form of tuples:
+        - Value in Position 0 is schema
+        - Value in Position 1 is an array of fingerprints:
+            - Position 0 is CRC-64-AVRO fingerprint
+            - Position 0 is MD5 fingerprint
+            - Position 0 is SHA256 fingerprint
+    """
+
+    def __init__(self, test_schema, fingerprints):
+        """Ignore the normal signature for unittest.TestCase because we are generating
+        many test cases from this one class. This is safe as long as the autoloader
+        ignores this class. The autoloader will ignore this class as long as it has
+        no methods starting with `test_`.
+        """
+        super(FingerprintTestCase, self).__init__("validate_fingerprint")
+        self.test_schema = test_schema
+        self.fingerprints = fingerprints
+
+    def _hex_fingerprint(self, fingerprint):
+        return "".join(format(b, "02x") for b in fingerprint).zfill(16)
+
+    def validate_fingerprint(self):
+        """The string of a Schema should be parseable to the same Schema."""
+        s = avro.schema.parse(self.test_schema)
+        self.assertEqual(self._hex_fingerprint(s.fingerprint()), self.fingerprints[0])
+        self.assertEqual(self._hex_fingerprint(s.fingerprint("md5")), self.fingerprints[1])
+        self.assertEqual(self._hex_fingerprint(s.fingerprint("sha256")), self.fingerprints[2])
+
+
 def load_tests(loader, default_tests, pattern):
     """Generate test cases across many test schema."""
     suite = unittest.TestSuite()
@@ -1262,6 +1458,8 @@ def load_tests(loader, default_tests, pattern):
     suite.addTests(DocAttributesTestCase(ex) for ex in DOC_EXAMPLES)
     suite.addTests(OtherAttributesTestCase(ex) for ex in OTHER_PROP_EXAMPLES)
     suite.addTests(loader.loadTestsFromTestCase(CanonicalFormTestCase))
+    suite.addTests(FingerprintTestCase(ex[0], ex[1]) for ex in FINGERPRINT_EXAMPLES)
+    suite.addTests(HashableTestCase(ex) for ex in VALID_EXAMPLES)
     return suite
 
 
